classdef CEnergyIOTemperatureModel < handle
    %ENERGYIOTEMPERATUREMODEL Energy-Temperature correlation model based on
    %Indoor/Outdoor temperature change and energy sequence.
    %   T[n+1] = k0 + k1*To[n] + k2*P[n] + T[n]
    %   Let x = exp(-lambdaA*TIME_GAP/mc)
    %   Let average outdoor temperature Tao = y0+ y1*To[n]
    %   T[n+1] = (1-x)y0 + (1-x)y1*To[n] + (1-x)r*P/lambdaA + x*T[n]
    
    properties
        m_A = 0;
        m_LambdaA = 0;
        m_y0 = 0;
        m_y1 = 0;
        m_mC = 0;
        m_r =0;
        k0 = 0;
        k1 = 0;
        k2 = 0;
        k3 = 0;
        
        m_timeGap;
        m_timeSeq;
        m_indoorTempSeq;
        m_denoisedIndoorTempSeq;
        m_denoisedEnergySeq;
        m_outdoorTempSeq;
        m_energySeq;
    end
    
    methods
       %% Constructor
        function obj = CEnergyIOTemperatureModel(roomSize, timeSeq, indoorTempSeq, outdoorTempSeq, energySeq)
            obj.m_timeSeq = timeSeq;
            obj.m_indoorTempSeq = indoorTempSeq;
            obj.m_outdoorTempSeq = outdoorTempSeq;
            obj.m_energySeq = energySeq;
            
            length = roomSize(1); width = roomSize(2); height = roomSize(3);
            obj.m_mC = (height*length*width)*1.2*1005;
            obj.m_A = length*width*2 + 2*(length+width)*height;
            
            obj.m_timeGap = timeSeq(1);
            %% NOTICE: Right now we assume the interval between items of sequence are constant
            obj.m_denoisedIndoorTempSeq = CEnergyIOTemperatureModel.denoiseTempData(obj.m_timeGap, obj.m_indoorTempSeq);
            obj.m_outdoorTempSeq = CEnergyIOTemperatureModel.denoiseTempData(obj.m_timeGap, obj.m_outdoorTempSeq);
            obj.m_denoisedEnergySeq = obj.m_energySeq;
            obj.calculateParameters();
        end
        
        %% Calculate temperature change use outdoor temp seq and energy seq
        function IndoorTempSeq = calculateTemperatureChange(obj, time_seq,...
                        start_indoor_temp, outdoorTempSeq, energySeq)
             current_temp = start_indoor_temp;
             seq_size = size(energySeq, 2);
             IndoorTempSeq = zeros(1, seq_size);
             IndoorTempSeq(1) = current_temp;
             for index = 1: seq_size-1
                 current_temp = obj.tempchange(time_seq(index), current_temp, outdoorTempSeq(index), energySeq(index));
                 IndoorTempSeq(index+1) = current_temp;
             end
                    
        end
        
        %% Calculate energy consumption use indoor and outdoor temp seq
        function energySeq = calculateEnergySeq(obj, time_seq, indoorTempSeq, outdoorTempSeq)
            seq_size = size(indoorTempSeq, 2);
            energySeq = zeros(1, seq_size);
            for index = 1: seq_size-1
                energySeq(index) = obj.averagePower( time_seq(index), indoorTempSeq(index), indoorTempSeq(index+1), outdoorTempSeq(index));
            end
            energySeq(seq_size) = energySeq(seq_size -1);
        end
        
        %% Temperature change calculation
        % time_length: minute
        function endTemp = tempchange(obj, time_length, Ts, To, power)
            endTemp = obj.k0 + obj.k1*To + obj.k2*power + obj.k3*Ts;
        end
        
        %% Average power calculation
        % Ts: start temperature
        % Te: End temperature
        % To: Outdoor temperature
        function avgPower = averagePower(obj, time_length, Ts, Te, To)
            avgPower = (Te-(obj.k0+obj.k1*To+obj.k3*Ts))/obj.k2;
        end
        
        %% Calculate the parameters
        function obj = calculateParameters(obj)
            seq_size = size(obj.m_energySeq, 2) -1;
            A = ones(seq_size, 4);
            A(:,2) = obj.m_outdoorTempSeq(1:seq_size);
            A(:,3) = obj.m_denoisedEnergySeq(1:seq_size);
            A(:,4) = obj.m_denoisedIndoorTempSeq(1:seq_size);
            b = (obj.m_denoisedIndoorTempSeq(2:(seq_size+1)))';
            x = (A'*A)\A'*b;
            obj.k0 = x(1);
            obj.k1 = x(2);
            obj.k2 = x(3);
            obj.k3 = x(4);
            
            
            % Calculate lambdaA
            % k3 = x = exp(-lambdaA*TIME_GAP/mC)
            obj.m_LambdaA = -log(obj.k3)*obj.m_mC/(obj.m_timeGap*60);
            % k2 = (1-x)*r/lambdaA
            obj.m_r = obj.k2*obj.m_LambdaA/(1-obj.k3);
            % k1 = (1-x)*y1;
            obj.m_y1 = obj.k1/(1-obj.k3);
            % k0 = (1-x)*y0;
            obj.m_y0 = obj.k0/(1-obj.k3);
        end
    end
    
    methods(Static)
        %% Denoise the indoor temperature data
        function denoisedTempSeq = denoiseTempData(timeGap, indoorTempSeq)
            
            real_temper_data = zeros(1,1);
            time_data = zeros(1,1);
            real_data_number = 0;
            array_size = size(indoorTempSeq, 2);
            node_num = size(indoorTempSeq, 1);
            for(data_index = 1:array_size)
                for(index = 1:node_num)
                    node_index = node_num +1- index;
                    if(indoorTempSeq(node_index,data_index) < 40)
                        real_data_number = real_data_number+1;
                        real_temper_data(real_data_number) = indoorTempSeq(node_index,data_index);
                        time_data(real_data_number) = 60*(timeGap*(data_index-1)+(index-1)*0.2);
                    end
                end
            end

            temper_data = real_temper_data';
            level = 5; 
            wname = 'sym4'; 
            tptr  = 'sqtwolog'; 
            sorh  = 's';
            npc_app = 1; 
            npc_fin = 1;
            denoise_data = wmulden(temper_data, level, wname, npc_app, npc_fin, tptr, sorh);
            denoise_data = denoise_data';
            
            %% Return the data with the same size
            denoisedTempSeq = denoise_data(1:node_num:(array_size*node_num));
        end
        

        function denoisedEnergySeq = denoiseEnergyData(energySeq, numOfTempNode)
            denoisedEnergySeq = energySeq;
            if numOfTempNode ~=1
                %The temperature data is denoised, energy should use
                %average
                seq_size = size(energySeq, 2);
                power_avg = mean(energySeq);
                period_index = 1;
                periods = zeros(3,2); % 1 vacancy period; 2 recooling period; 3 maintain period
                periods(period_index,1) = 1;
                for (index = 1:seq_size)
                    power = energySeq(index);
                    if (power > power_avg) && (period_index == 1)
                        periods(period_index, 2)= index-1;
                        period_index = 2;
                        periods(period_index,1) = index;
                    end

                    if(power < power_avg) && (period_index == 2)
                        periods(period_index, 2) = index-1;
                        period_index = 3;
                        periods(period_index, 1) = index;
                        periods(period_index, 2) = seq_size;
                        break;
                    end
                end
                
                mp_start = periods(3, 1);
                mp_end = periods(3, 2);
                mp_avg_power = mean(energySeq(mp_start:mp_end));
                denoisedEnergySeq(mp_start:mp_end) = ones(1, mp_end-mp_start+1)*mp_avg_power;
            end
        end

    end
    
end

